/*++

Copyright (c) 2004  Future Technology Devices International Ltd.

Module Name:

    morphprg.h

Abstract:

    API DLL for FPGA MorphIC Type Devices.
    MORPHPRG library definitions

Environment:

    kernel & user mode

Revision History:

    15/03/04    kra     Created.
	
--*/


#ifndef MORPHPRG_H
#define MORPHPRG_H


// The following ifdef block is the standard way of creating macros
// which make exporting from a DLL simpler.  All files within this DLL
// are compiled with the MORPHPRG_EXPORTS symbol defined on the command line.
// This symbol should not be defined on any project that uses this DLL.
// This way any other project whose source files include this file see
// MORPHPRG_API functions as being imported from a DLL, whereas this DLL
// sees symbols defined with this macro as being exported.

#ifdef MORPHPRG_EXPORTS
#define MORPHPRG_API __declspec(dllexport)
#else
#define MORPHPRG_API __declspec(dllimport)
#endif

typedef DWORD MP_HANDLE;
typedef ULONG MP_STATUS;

#define MP_SUCCESS 0  //FT_OK
#define MP_INVALID_HANDLE 1 //FT_INVALID_HANDLE
#define MP_DEVICE_NOT_FOUND 2 //FT_DEVICE_NOT_FOUND
#define MP_DEVICE_NOT_OPENED 3 //FT_DEVICE_NOT_OPENED
#define MP_IO_ERROR 4 //FT_IO_ERROR
#define MP_INSUFFICIENT_RESOURCES 5 //FT_INSUFFICIENT_RESOURCES

#define MP_DEVICE_NAME_BUFFER_TOO_SMALL 20
#define MP_INVALID_DEVICE_NAME 21
#define MP_INVALID_DEVICE_INDEX 22
#define MP_TOO_MANY_DEVICES 23
#define MP_DEVICE_IN_USE 24
#define MP_FAILED_TO_COMPLETE_COMMAND 25
#define MP_FAILED_TO_CLEAR_DEVICE 26
#define MP_FAILED_TO_PROGRAM_DEVICE 27
#define MP_FAILED_TO_SYNCHRONIZE_DEVICE 28
#define MP_SERIAL_NUMBER_BUFFER_TOO_SMALL 29
#define MP_DLL_VERSION_BUFFER_TOO_SMALL 30
#define MP_INVALID_LANGUAGE 31
#define MP_INVALID_STATUS_CODE 32
#define MP_ERROR_MESSAGE_BUFFER_TOO_SMALL 33
#define MP_INVALID_SERIAL_NUMBER 34
#define MP_INVALID_SERIAL_NUMBER_INDEX 35
#define MP_INVALID_FILE_NAME 36
#define MP_INVALID_FILE_EXTENSION 37
#define MP_FILE_NOT_FOUND 38
#define MP_FAILED_TO_OPEN_FILE 39
#define MP_FILE_READ_ERROR 40
#define MP_FILE_CORRUPT 41


#ifdef __cplusplus
extern "C" {
#endif

MORPHPRG_API
MP_STATUS WINAPI GetNumMorphICDevices(LPDWORD lpdwNumMorphICDevices);

MORPHPRG_API
MP_STATUS WINAPI GetMorphICDeviceName(DWORD dwDeviceIndex, LPSTR lpDeviceNameBuffer, DWORD dwBufferSize);

MORPHPRG_API
MP_STATUS WINAPI GetNumMorphICDeviceSerialNumbers(LPSTR lpDeviceName, LPDWORD lpdwNumMorphICDeviceSerialNumbers);

MORPHPRG_API
MP_STATUS WINAPI GetMorphICDeviceSerialNumber(LPSTR lpDeviceName, DWORD dwSerialNumberIndex, LPSTR lpSerialNumberBuffer, DWORD dwBufferSize);

MORPHPRG_API
MP_STATUS WINAPI OpenSpecifiedMorphICDevice(LPSTR lpDeviceName, LPSTR lpSerialNumber, MP_HANDLE *pMpHandle);

MORPHPRG_API
MP_STATUS WINAPI OpenMorphICDevice(MP_HANDLE *pMpHandle);

MORPHPRG_API
MP_STATUS WINAPI CloseMorphICDevice(MP_HANDLE MpHandle);

MORPHPRG_API
MP_STATUS WINAPI InitMorphICDevice(MP_HANDLE MpHandle);

MORPHPRG_API
MP_STATUS WINAPI ProgramMorphICDevice(MP_HANDLE MpHandle, LPSTR lpFileName);

MORPHPRG_API
MP_STATUS WINAPI IsMorphICDeviceProgrammed(MP_HANDLE MpHandle, LPBOOL lpbProgramState);

MORPHPRG_API
MP_STATUS WINAPI GetMorphICDllVersion(LPSTR lpDllVersionBuffer, DWORD dwBufferSize);

MORPHPRG_API
MP_STATUS WINAPI GetMorphICErrorCodeString(LPSTR lpLanguage, MP_STATUS StatusCode, LPSTR lpErrorMessageBuffer, DWORD dwBufferSize);

#ifdef __cplusplus
}
#endif


#endif  /* MORPHPRG_H */