unit DSjtag;

//====================================================
//==
//== This block will program the FPGA or setup the
//== serial bit stream to control the scope. It will
//== restore the device handle on exit
//==
//==
//== bit 0 - DCLK                out
//== bit 1 - DATA0               out
//== bit 2 - nconfig             out
//== bit 3 - nStatus             in
//== bit 4 - nConfDone           in
//== bit 5 -
//== bit 6 -
//== bit 7 - RSTIN               out
//==
//====================================================

interface

Uses Windows,sysutils,D2XXUNIT;

Type
  Names = Array[1..20] of string;
  Names_Ptr = ^Names;



// Exported Functions
Function Program_FPGA(DevName : string;RBFFileName : string ) : boolean;

Function Dev_Configured(DevName : string) : boolean;

Function Reset_Function(DevName : string) : boolean;

procedure List_Devs( My_Names_Ptr : Names_Ptr);

Var
  Saved_Handle: DWord;
  PortAIsOpen : boolean;
  BlockData : Array[0..999] of Byte;
  OffSet : integer;
  OutIndex : integer;
  PortState : Byte;
  PageData : Array[0..511] of Byte;

  //  RBFFileName : TfileName;

const USBBuffSize : integer = $4000;


implementation

procedure DisplayMsg(MsgStr : string);
begin
//mainform.memo1.lines.add(msgstr);
end;

function OpenPort(PortName : string) : boolean;
Var res : FT_Result;
NoOfDevs,i,J : integer;
Name : String;
DualName : string;
done : boolean;
begin
PortAIsOpen := False;
OpenPort := False;
Name := '';
Dualname := PortName; //  + ' A';
//Dualname := 'Dual Device ' + extension;
res := GetFTDeviceCount;
if res <> Ft_OK then exit;
NoOfDevs := FT_Device_Count;
j := 0;
if NoOfDevs > 0 then
  begin
    repeat
      repeat
      res := GetFTDeviceDescription(J);
      if (res <> Ft_OK) then  J := J + 1;
      until (res = Ft_OK) OR (J=NoOfDevs);
    if res <> Ft_OK then exit;
    done := false;
    i := 1;
    Name := '';
      repeat
      if ORD(FT_Device_String_Buffer[i]) <> 0 then
        begin
        Name := Name + FT_Device_String_Buffer[i];
        OpenPort := TRUE;
        end
      else
        begin
        done := true;
        OpenPort := FALSE;
        end;
      i := i + 1;
      until done;
    J := J + 1;
    until (J = NoOfDevs) or (name = DualName);
  end;

if (name = DualName) then
  begin
  res := Open_USB_Device_By_Device_Description(name);
  if res <> Ft_OK then exit;
  OpenPort := true;
  res := Get_USB_Device_QueueStatus;
  if res <> Ft_OK then exit;
  PortAIsOpen := true;
  end
else
  begin
//  MessageDlg('Could Not Open '+ name, mtError, [mbOk], 0);
  OpenPort := false;
  end;

end;


procedure ClosePort;
Var res : FT_Result;
begin
if PortAIsOpen then
  res := Close_USB_Device;
PortAIsOpen := False;
end;


procedure SendBytes(NumberOfBytes : integer);
var i : integer;
begin
i := Write_USB_Device_Buffer( NumberOfBytes);
outindex := outindex - i;
end;

procedure SetupPort;
var res : FT_Result;

begin
res := Reset_USB_Device;
res := Reset_USB_Device;
//res := Set_USB_Parameters(USBBuffSize,0);
end;



function ReceiveBuffer (RxCount : integer)  : integer;
var i : integer;
Var res : FT_Result;
begin
Result := 0;
res := Get_USB_Device_QueueStatus;
IF (FT_Q_Bytes > 0) then
  begin
  if (RxCount <> 0) then
    begin
    if (RxCount > FT_Q_Bytes) then
      begin
      i := Read_USB_Device_Buffer(FT_Q_Bytes);
      result := i;
      end
    else
      begin
      i := Read_USB_Device_Buffer(RxCount);
      result := i;
      end;
    end;
  end;
end;


procedure List_Devs( My_Names_Ptr : Names_Ptr);
Var res : FT_Result;
NoOfDevs,i,J,k : integer;
Name : String;
done : boolean;
begin
PortAIsOpen := False;
Name := '';
res := GetFTDeviceCount;
if res <> Ft_OK then exit;
NoOfDevs := FT_Device_Count;
j := 0;
k := 1;
if NoOfDevs > 0 then
  begin
    repeat
    res := GetFTDeviceDescription(J);
    if res = Ft_OK then
      begin
      done := false;
      i := 1;
      Name := '';
        repeat
        if ORD(FT_Device_String_Buffer[i]) <> 0 then
          begin
          Name := Name + FT_Device_String_Buffer[i];
          end
        else
          begin
          done := true;
          My_Names_Ptr[k]:= Name;
          k := k + 1;
          end;
        i := i + 1;
        until done;
      end;
    J := J + 1;
    until (J = NoOfDevs);
  end;

end;

procedure AddToBuffer(I:integer);
begin
FT_Out_Buffer[OutIndex]:= I AND $FF;
inc(OutIndex);
end;


function timeout_ms(start_time, end_time: TDateTime; timeout_val: integer): boolean;
var
  start_hour, start_min, start_sec, start_ms: word;
  end_hour,   end_min,   end_sec,   end_ms  : word;
begin
  decodetime(start_time, start_hour, start_min, start_sec, start_ms);
  decodetime(end_time,   end_hour,   end_min,   end_sec,   end_ms);
  end_min := end_min + (60 * (end_hour - start_hour));
  end_sec := end_sec + (60 * (end_min - start_min));
  end_ms  := end_ms  + (1000 * (end_sec - start_sec));
  timeout_ms := (end_ms - start_ms) >= timeout_val;
end;


function CheckStatus : boolean;
// returns true unless there is an error
var res : FT_Result;
NStat : byte;
i : integer;
timeout_start: TDateTime;
timeout_end  : TDateTime;
timeout     : boolean;
begin
CheckStatus := false;
res := Get_USB_Device_QueueStatus;
IF (FT_Q_Bytes > 0) then
      i := Read_USB_Device_Buffer(FT_Q_Bytes);
AddToBuffer($81); // direction
AddToBuffer($87); // send imm
SendBytes(OutIndex);
OutIndex := 0;
timeout_start := now;
  repeat
  res := Get_USB_Device_QueueStatus;
  if FT_Q_Bytes = 0 then sleep(0);
  timeout_end := now;
  timeout := timeout_ms(timeout_start, timeout_end, 5000);
  until (FT_Q_Bytes > 0) or (res <> FT_OK) or timeout; // or timeout
if (res <> FT_OK) or timeout then exit;
i := Read_USB_Device_Buffer(FT_Q_Bytes);
NStat := FT_In_Buffer[0];
NStat := NStat AND $08;
if NStat <> 0 then CheckStatus := true;
end;


function Sync_To_MPSSE : boolean;
//
// This should satisfy outstanding commands.
//
// We will use $AA and $AB as commands which
// are invalid so that the MPSSE block should echo these
// back to us preceded with an $FA
//
var res : FT_Result;
i,j : integer;
Done,failed : boolean;
timeout_start: TDateTime;
timeout_end  : TDateTime;
timeout     : boolean;
begin
Sync_To_MPSSE := false;
failed := false;
done := false;
res := Get_USB_Device_QueueStatus;
if res <> FT_OK then exit;
if (FT_Q_Bytes > 0) then i := Read_USB_Device_Buffer(FT_Q_Bytes);

timeout_start := now;

repeat
  repeat
    OutIndex := 0;
    AddToBuffer($AA); // bad command
    SendBytes(OutIndex);
    res := Get_USB_Device_QueueStatus;
    if FT_Q_Bytes = 0 then sleep(0);
    timeout_end := now;
    timeout := timeout_ms(timeout_start, timeout_end, 1000);
  until (FT_Q_Bytes > 0) or (res <> FT_OK) or timeout;
  if res <> FT_OK then failed := true;
  if (FT_Q_Bytes > 0) then
    begin
//    MessageDlg('Expected bytes '+inttostr(FT_Q_Bytes), mtInformation, [mbOk], 0);
    i := Read_USB_Device_Buffer(FT_Q_Bytes);
    if (FT_In_Buffer[i-1] = $AA) then Done := true;
    end;
until done or failed or timeout;

done := false;

if failed or timeout then
  begin
//  closeport;
  end
else
  begin
  OutIndex := 0;
  AddToBuffer($AB); // bad command
  SendBytes(OutIndex);
  timeout_start := now;
  repeat
    repeat
      res := Get_USB_Device_QueueStatus;
      if (FT_Q_Bytes=0) then sleep(0);
      timeout_end := now;
      timeout := timeout_ms(timeout_start, timeout_end, 1000);
    until (FT_Q_Bytes > 0) or (res <> FT_OK) or timeout;
    if res <> FT_OK then exit;
    if (FT_Q_Bytes > 0) then
      begin
      i := Read_USB_Device_Buffer(FT_Q_Bytes);
      j := 0;
      repeat
      if (FT_In_Buffer[j] = $FA) then
        begin
        if (j <= (i-2)) then
          begin
          if (FT_In_Buffer[j+1] = $AB) then Done := true;
          end;
        end;
      j := j + 1;
      until (j=i) or Done;
      end;
  until done or timeout;

  if Done then Sync_To_MPSSE := true;
  end;
end;



{

// Old Function

function Sync_To_MPSSE : boolean;
//
// This should satisfy outstanding commands.
//
// We will use $AA and $AB as commands which
// are invalid so that the MPSSE block should echo these
// back to us preceded with an $FA
//

var res : FT_Result;
i,j : integer;
Done : boolean;
timeout_start: TDateTime;
timeout_end  : TDateTime;
timeout     : boolean;
begin
Sync_To_MPSSE := false;
res := Get_USB_Device_QueueStatus;
if res <> FT_OK then exit;
if (FT_Q_Bytes > 0) then
  i := Read_USB_Device_Buffer(FT_Q_Bytes);
repeat
timeout_start := now;
  repeat
  OutIndex := 0;
  AddToBuffer($AA); // bad command
  SendBytes(OutIndex);
  res := Get_USB_Device_QueueStatus;
  if FT_Q_Bytes = 0 then sleep(0);
  timeout_end := now;
  timeout := timeout_ms(timeout_start, timeout_end, 5000);
  until (FT_Q_Bytes > 0) or (res <> FT_OK) or timeout; // or timeout

  if (res <> FT_OK) or timeout then exit;
  i := Read_USB_Device_Buffer(FT_Q_Bytes);
  j := 0;
  Done := False;
    repeat
    if (FT_In_Buffer[j] = $FA) then
      begin
      if (j <= (i-2)) then
        begin
        if (FT_In_Buffer[j+1] = $AA) then Done := true;
        end;
      end;
    j := j + 1;
    until (j=i) or Done;
until Done or (res <> FT_OK) or timeout; // or timeout
if (res <> FT_OK) or timeout then exit;

OutIndex := 0;
AddToBuffer($AB); // bad command
SendBytes(OutIndex);
timeout_start := now;
  repeat
  res := Get_USB_Device_QueueStatus;
  if FT_Q_Bytes = 0 then sleep(0);
  timeout_end := now;
  timeout := timeout_ms(timeout_start, timeout_end, 5000);
  until (FT_Q_Bytes > 0) or (res <> FT_OK) or timeout; // or timeout
if (res <> FT_OK) or timeout then exit;
i := Read_USB_Device_Buffer(FT_Q_Bytes);
j := 0;
Done := False;
  repeat
  if (FT_In_Buffer[j] = $FA) then
    begin
    if (j <= (i-2)) then
      begin
      if (FT_In_Buffer[j+1] = $AB) then Done := true;
      end;
    end;
  j := j + 1;
  until (j=i) or Done;

if Done then Sync_To_MPSSE := true;
end;
}


procedure SendBlock(Size : integer);
var i,j : integer;
begin
AddToBuffer($19); // send bytes
j := size - 1;
i := j mod 256;
AddToBuffer(i); // low size
i := (j div 256) AND $FF;
AddToBuffer(i); // high size
for i := 0 to j do
  AddToBuffer(PageData[i]);
SendBytes(OutIndex);
OutIndex := 0;
end;

procedure SendBitByte(ByteToSnd : byte; SendNow : Boolean );
var SendingByte : byte;
begin
FT_Out_Buffer[Offset]:= ByteToSnd;
OffSet := OffSet + 1;
if (SendNow OR (OffSet = 4096)) then
  begin
  SendBlock(OffSet);
  OffSet := 0;
  end;

end;

procedure CompleteProg;
var SendingByte : byte;
begin
  AddToBuffer($19); // clk data out on -ve edge LSB
  AddToBuffer($01); // 2 bytes
  AddToBuffer($00);
  AddToBuffer($06);
  AddToBuffer($06);
  SendBytes(OutIndex);
  OutIndex := 0;
end;

function CheckDone : boolean;
// returns false unless it is finished
var res : FT_Result;
NStat : byte;
i : integer;
timeout_start: TDateTime;
timeout_end  : TDateTime;
timeout     : boolean;
begin
CheckDone := false;
res := Get_USB_Device_QueueStatus;
IF (FT_Q_Bytes > 0) then
      i := Read_USB_Device_Buffer(FT_Q_Bytes);
AddToBuffer($81); // direction
AddToBuffer($87); // send imm
SendBytes(OutIndex);
OutIndex := 0;
timeout_start := now;
  repeat
  res := Get_USB_Device_QueueStatus;
  if FT_Q_Bytes = 0 then sleep(0);
  timeout_end := now;
  timeout := timeout_ms(timeout_start, timeout_end, 5000);
  until (FT_Q_Bytes > 0) or (res <> FT_OK) or timeout; // or timeout
if (res <> FT_OK) or timeout then exit;
i := Read_USB_Device_Buffer(FT_Q_Bytes);
NStat := FT_In_Buffer[0];
NStat := NStat AND $10;
if NStat <> 0 then CheckDone := true;
end;

function StartConfig : boolean;
// returns true if confdone is low after pulsing nConfig
var started : boolean;
begin
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($06); // value
  AddToBuffer($87); // direction
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($02); // value
  AddToBuffer($87); // direction
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($06); // value
  AddToBuffer($87); // direction
  SendBytes(OutIndex);
  OutIndex := 0;
  started := CheckDone;
  if started then StartConfig := False else StartConfig := True;
end;

procedure Add0;
var i : integer;
begin
i := PortState;
AddToBuffer(i);
i := i OR $40;
AddToBuffer(i);
end;

procedure Add1;
var i : integer;
begin
i := PortState OR $20;
AddToBuffer(i);
i := i OR $40;
AddToBuffer(i);
end;


Function Program_FPGA(DevName : string;RBFFileName : string ) : boolean;
var RBFFile : file;
iBytesRead,BytesWritten : integer;
i : integer;
Complete,Passed,ImmSend : Boolean;
res : FT_Result;
begin // 1
Saved_Handle := FT_Handle;
Program_FPGA := false;
if RBFFileName <> '' then
  begin  //2
  // Bit Mode On
  passed := OpenPort(DevName);
  if passed then
    begin //3
    SetupPort;
    res := Set_USB_Device_BitMode($00,$02);
    passed := Sync_To_MPSSE;
    end; //3
  if NOT passed then
    begin //3
    ClosePort;
    end  //3
  else
    begin //3
    OutIndex := 0;
    AddToBuffer($86); // set clk to 6MHz
    AddToBuffer($00);
    AddToBuffer($00);
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($06); // value
    AddToBuffer($87); // direction
    SendBytes(OutIndex);
    OutIndex := 0;

    AssignFile(RBFFile, RBFFileName);	// Open input file
    Reset(RBFFile,1);	                // Record size = 1
    Complete := false;
    OffSet := 0;
    res := Reset_USB_Device;
    FT_Current_Baud := 3000000;
    res := Set_USB_Device_BaudRate;
    BytesWritten := 0;

    if not( EOF(RBFFile)) then
      begin  //4
      Complete := StartConfig; // start sequence - will return success
      if Complete then
        begin   //5
        if Complete then Complete := False else Complete := True;
        repeat
        BlockRead(RBFFile, PageData, 512, iBytesRead);
        if (iBytesRead > 0) then
          begin  //6
          SendBlock(iBytesRead);
          end;   //6
          Complete := CheckDone;
          Passed := CheckStatus;
  //      DisplayMsg(' Bytes Written = '+inttostr(BytesWritten));
        until (iBytesRead = 0) OR Complete OR NOT Passed;
        end;  //5
      end; //4
    if (iBytesRead = 0) then
      begin //4
      end;  //4
    if complete then
      begin //4
      CompleteProg; // for last 10 clocks
      DisplayMsg(' Programming Passed ');
      Program_FPGA := true;
      end  //4
    else
      begin  //4
      if NOT Passed then
        begin  //5
        DisplayMsg(' Programming Failed - nStatus');
    //    DisplayMsg(' Bytes Written = '+ inttostr(BytesWritten));
        end   //5
      else
        begin //5
        DisplayMsg(' Programming Failed - ran out of file');
    //    DisplayMsg(' Bytes Written = '+ inttostr(BytesWritten));
        CompleteProg; // for last 10 clocks
        end; //5
      end;  //4
    CloseFile(RBFFile);

    // reset device by pulsing bit 7
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($06); // value
    AddToBuffer($87); // direction
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($86); // value
    AddToBuffer($87); // direction
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($06); // value
    AddToBuffer($87); // direction
    SendBytes(OutIndex);
    OutIndex := 0;

    ClosePort;
    end  //3
  end  //3
else
  begin //2
  DisplayMsg(' You must select a file');
  end; //2

FT_Handle := Saved_Handle;

end; //1


Function old_Program_FPGA(DevName : string;RBFFileName : string ) : boolean;
var RBFFile : file;
iBytesRead,BytesWritten : integer;
i : integer;
Complete,Passed,ImmSend : Boolean;
res : FT_Result;
begin
Saved_Handle := FT_Handle;
old_Program_FPGA := false;
if RBFFileName <> '' then
  begin
// Bit Mode On
passed := OpenPort(DevName);
if passed then
  begin
  SetupPort;
  res := Set_USB_Device_BitMode($00,$02);
  OutIndex := 0;
  AddToBuffer($86); // set clk to 6MHz
  AddToBuffer($00);
  AddToBuffer($00);
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($06); // value
  AddToBuffer($87); // direction
  SendBytes(OutIndex);
  OutIndex := 0;



  AssignFile(RBFFile, RBFFileName);	// Open input file
  Reset(RBFFile,1);	                // Record size = 1
  Complete := false;
  OffSet := 0;
  res := Reset_USB_Device;
  FT_Current_Baud := 2000000;
  res := Set_USB_Device_BaudRate;
  BytesWritten := 0;

  if not( EOF(RBFFile)) then
    begin
    Complete := StartConfig; // start sequence - will return success
    if Complete then
      begin
      if Complete then Complete := False else Complete := True;
      repeat
      BlockRead(RBFFile, PageData, 512, iBytesRead);
      if (iBytesRead > 0) then
        begin
        SendBlock(iBytesRead);
        end;
        Complete := CheckDone;
        Passed := CheckStatus;
//      DisplayMsg(' Bytes Written = '+inttostr(BytesWritten));
      until (iBytesRead = 0) OR Complete OR NOT Passed;
    end;
  end;
  if (iBytesRead = 0) then
    begin
    end;
  if complete then
    begin
    CompleteProg; // for last 10 clocks
    DisplayMsg(' Programming Passed ');
    old_Program_FPGA := true;
    end
  else
    begin
    if NOT Passed then
      begin
      DisplayMsg(' Programming Failed - nStatus');
  //    DisplayMsg(' Bytes Written = '+ inttostr(BytesWritten));
     end
    else
      begin
      DisplayMsg(' Programming Failed - ran out of file');
  //    DisplayMsg(' Bytes Written = '+ inttostr(BytesWritten));
      CompleteProg; // for last 10 clocks
      end;
    end;
  CloseFile(RBFFile);

  // reset device by pulsing bit 7
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($06); // value
  AddToBuffer($87); // direction
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($86); // value
  AddToBuffer($87); // direction
  AddToBuffer($80); // Set data bits low byte
  AddToBuffer($06); // value
  AddToBuffer($87); // direction
  SendBytes(OutIndex);
  OutIndex := 0;

  ClosePort;
    end
  else
    DisplayMsg(' You must select a file');
  end;

FT_Handle := Saved_Handle;

end;

Function Dev_Configured(DevName : string) : boolean;
var passed : boolean;
res : FT_Result;
begin
Saved_Handle := FT_Handle;
Dev_Configured := false;

// Bit Mode On
passed := OpenPort(DevName);
if passed then
  begin
  OutIndex := 0;
  SetupPort;
  res := Set_USB_Device_BitMode($00,$02);
  if res = FT_OK then
    begin
    res := Reset_USB_Device;
    end;
  if res = FT_OK then
    begin
    passed := Sync_To_MPSSE;
    end;
  end;
if NOT passed then
  begin
  ClosePort;
  end
else
  begin
  passed := CheckDone;
  if passed then Dev_Configured := true;
  closeport;
  end;

FT_Handle := Saved_Handle;
end;

Function Reset_Function(DevName : string) : boolean;
var passed : boolean;
res : FT_Result;
begin
Saved_Handle := FT_Handle;
Reset_Function := false;

// Bit Mode On
passed := OpenPort(DevName);
if passed then
  begin
  OutIndex := 0;
  SetupPort;
  res := Set_USB_Device_BitMode($00,$02);
  if res = FT_OK then
    begin
    res := Reset_USB_Device;
    end;
  if res = FT_OK then
    begin
    passed := Sync_To_MPSSE;
    end;
  if passed then
    begin
    passed := CheckDone;
    end;
  if passed then
    begin
    Reset_Function := true;
    // reset device by pulsing bit 7
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($06); // value
    AddToBuffer($87); // direction
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($86); // value
    AddToBuffer($87); // direction
    AddToBuffer($80); // Set data bits low byte
    AddToBuffer($06); // value
    AddToBuffer($87); // direction
    SendBytes(OutIndex);
    OutIndex := 0;
    end;
  closeport;
  end;

FT_Handle := Saved_Handle;
end;


end.
